<?php

namespace Drupal\site_studio_master_template_selector\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines the 'site_studio_master_template_selector_widget' field widget.
 *
 * @FieldWidget(
 *   id = "site_studio_master_template_selector_widget",
 *   label = @Translation("Site Studio master template selector widget"),
 *   field_types = {
 *     "cohesion_template_selector"
 *   }
 * )
 */
class SiteStudioMasterTemplateSelectorWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    // Get the entity this form is working on.
    if (method_exists($items, 'getParent')) {
      $entity = $items->getParent()->getValue();
    }
    else {
      return [];
    }

    // Get list of templates for this content type.
    $template_ids = \Drupal::service('entity_type.manager')->getStorage('cohesion_master_templates')->getQuery()
      ->condition('status', TRUE)
      ->condition('selectable', TRUE)
      ->sort('default', 'DESC')
      ->execute();

    $template_storage = \Drupal::entityTypeManager()->getStorage('cohesion_master_templates');
    $templates = $template_storage->loadMultiple($template_ids);

    $options = [
      '__default__' => t('Default master template'),
    ];

    foreach ($templates as $key => $template) {
      if ($template->status() || $items[$delta]->selected_template == $key) {
        $options[$key] = t('%label', ['%label' => $template->get('label')]);
      }
    }

    $element['selected_template'] = [
      '#type' => 'select',
      '#title' => t('Select template:'),
      '#description' => t('Select a master template that will be used to wrap this content.'),
      '#options' => $options,
      '#default_value' => (isset($options[$items[$delta]->selected_template])) ? $items[$delta]->selected_template : '__default__',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as &$item) {
      if (isset($item['selected_template']['content'])) {
        $item['selected_template'] = $item['selected_template']['content'];
      }
    }
    return $values;
  }

}
